/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * This is the panel where all the drawings are actually performed
 * @author ycao
 */
public class VinlGraphicPanel extends JPanel {

	private Image buf_img;
	
	private Graphics2D buf_g;

	private Dimension d;
	
	private VinlTurtle turtle;

	private Color penColor;

	/**
	 * Default Constructor
	 */
	public VinlGraphicPanel() {
	}

	public void init(Dimension d) {
		this.d = d;
		this.setSize(d);
		buf_img = this.createImage(d.width, d.height);
		buf_g = (Graphics2D)buf_img.getGraphics();
		penColor = Color.BLACK;
		buf_g.setColor(penColor);
		this.setBackground(Color.WHITE);
		turtle = new VinlTurtle(d.width/2, d.height/2, 90);
	}
	
	/**
	 * Draw a line length of r from the current turtle position 
	 * in the turtle facing direction 
	 * @param r -- distance from current point
	 */
	public void forward(double r) {
		double currX = turtle.getX();
		double currY = turtle.getY();
		double theta = turtle.getAngle() * Math.PI / 180;
		double x = currX + (Math.cos(theta) * r);
		//because y coordinate in Java goes down, do minus instead of plus here
		double y = currY - (Math.sin(theta) * r);
		if (turtle.isPenDown()) {
			//need to sacrafice some accuracy 
			buf_g.drawLine((int)currX, (int)currY, (int)x, (int)y);
			VinlAstWalker.log(String.format("coordinates: %f %f %f %f. %n", currX, currY, x, y), VinlAstWalker.DEBUG_FINE);
		}
		turtle.setX(x);
		turtle.setY(y);
	}

	/**
	 * Draw a line length of r from the current turtle position
	 * in the direction opposite of the turtle facing direction 
	 * @param r -- distance from current point
	 */
	public void backward(double r) {
		forward((-1)*r);
	}

	/**
	 * turn the turtle x degrees to the right 
	 * @param x -- in degrees
	 */
	public void right(double x) {
		turtle.setAngle(turtle.getAngle()-x);
	}
	
	/**
	 * turn the turtle x degrees to the right 
	 * @param x -- in degrees
	 */
	public void left(double x) {
		turtle.setAngle(turtle.getAngle()+x);
	}

	/**
	 * Clear the entire screen 
	 */
	public void clearScreen() {
		//empty the buffer and start a new
		this.init(d);
	}

	public void homeTurtle() {
		turtle = new VinlTurtle(d.width/2, d.height/2, 90);
	}

	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		g.drawImage(buf_img, 0, 0, this);
	}
	
	public void drawTurtle() {
		//create a triangle based on the current x, y and angle
		Polygon triangle = new Polygon();
		double theta = turtle.getAngle() * Math.PI / 180;
		double x = turtle.getX() + (Math.cos(theta) * 10);
		double y = turtle.getY() - (Math.sin(theta) * 10);
		triangle.addPoint((int)x, (int)y);
		triangle.addPoint((int)(turtle.getX()+(Math.sin(theta)*5)), (int)(turtle.getY()+(Math.cos(theta)*5)) );
		triangle.addPoint((int)(turtle.getX()-(Math.sin(theta)*5)), (int)(turtle.getY()-(Math.cos(theta)*5)) );
		buf_g.draw(triangle);
		turtle.setTriangle(triangle);
	}

	public void eraseTurtle() {
		Polygon triangle = turtle.getTriangle();
		if(triangle != null) {
			buf_g.setColor(buf_g.getBackground());
			buf_g.draw(triangle);
			buf_g.setColor(penColor);
		}
	}

	public VinlTurtle getTurtle() {
		return this.turtle;
	}
	
}
